function [X,p] = RX_rec_sym2full(X11,X12,X22,BxNN,ByNN,txN,tyN,posdeftest)
% constructs the MoM matrix for a rectangular plate from the elements of the first row
% use Calc_RX_matrices_rec_CurroptTutorial(k1,k2,kN,lx,ly,Nx,Ny,lib);
% to compute X11, X12, X22 and so on
%
% Mats Gustafsson 2010-15, http://www.eit.lth.se/staff/mats.gustafsson
%
% See 
% M. Gustafsson, D. Tayli, C. Ehrenborg, M. Cismasu, S. Nordebo
% Tutorial on antenna current optimization using MATLAB and CVX
% 2015
% and
% M. Gustafsson, S. Nordebo:  
% Optimal antenna currents for Q, superdirectivity, and radiation patterns using convex optimization
% IEEE Transactions on Antennas and Propagation, Vol. 61, No. 3, pp. 1109-1118, 2013.
% for additional information
%%


if min(txN,tyN) == 1
    if txN == 1
        X = toeplitz(X22,X22);
    else
        X = toeplitz(X11,X11);
    end
else
    Nx = BxNN*tyN;
    Ny = ByNN*txN;
    Nt = Nx+Ny;    
    X = zeros(Nt);    
    X(1:Nx,1:Nx) = imp1Dto2D(X11,BxNN,tyN);
    X(Nx+(1:Ny),Nx+(1:Ny)) = imp1Dto2D(X22,ByNN,txN);
    Z21r = imp1Dto2Dxy(X12,ByNN,txN,BxNN,tyN);
    X(Nx+(1:Ny),1:Nx) = Z21r;
    X(1:Nx,Nx+(1:Ny)) = Z21r.';
    X = (X+X.')/2;
end
if posdeftest
    [R,p] = chol(X);
    if (p>0 && posdeftest>1)
        [X,InX,sqrtX] = indef2possemdef(X,0);
        p = InX;
    end
else
    p = 0;
end
end

function Z11 = imp1Dto2D(Z1,bxN,tyN)

%Z1 = Z11s(1,:);
B = reshape(Z1,bxN,tyN);
Z11 = zeros(bxN*tyN);
for n=0:tyN-1
    for m=0:tyN-1
%         nn = mod(m+n,tyN);
%         Z11((m*bxN)+(1:bxN),nn*bxN+(1:bxN)) = toeplitz(B(:,n+1));
        %nn = mod(m+n,tyN);
        Z11((m*bxN)+(1:bxN),n*bxN+(1:bxN)) = toeplitz(B(:,abs(n-m)+1),B(:,abs(n-m)+1));
    end
end
Z11 = triu(Z11);
Z11 = Z11+Z11.'-diag(diag(Z11));
end

function Z12 = imp1Dto2Dxy(Z1,bxN,tyN,byN,txN)

B = reshape(Z1,byN,txN).';
Z12 = zeros(bxN*tyN,byN*txN);
T1 = toeplitz([1 1:tyN-1],[1:byN]);
T2 = toeplitz([1 (2:bxN)+1],[1:txN]);
for n=0:tyN-1
    for m=0:txN-1
        for p=0:byN-1
            for q=0:bxN-1
                sig = (-1)^(p<n)*(-1)^(m<q);
                Z12(n*bxN+q+1,(m*byN)+p+1) = sig*B(T2(q+1,m+1),T1(n+1,p+1));
            end
        end
    end
end
end

function [Z,InZ,sqrtZ] = indef2possemdef(Z,regeps)
% transforms a matrix to a symmetric positive semidefinite matrix by
% changing eigenvalues < regeps to regesp
% Also computes the square root sqrtZ*sqrtZ=Z
Z = (Z+Z')/2;
[V,D] = eig(Z);
D1 = diag(D);
[ind] = find(D1<regeps);
InZ = D1(ind);
D1(ind) = regeps;
D = diag(D1);
Z = V*D*V';
sqrtD = diag(sqrt(D1));
sqrtZ = V*sqrtD*V';

end
